;;;; doc.scm
;;;; :tabSize=2:indentSize=2:noTabs=true:
;;;; Documentation of the SQLite3 bindings

(use eggdoc)

(define doc '(
  (eggdoc:begin

    (name "sqlite3")
    (description (p "Bindings to version 3.x of the SQLite API."))
    (author
      (url "http://www.chust.org/" "Thomas Chust"))
    (usage)
    (download "sqlite3.egg")
    (requires "check-errors" "synch" "miscmacros" "matchable" "sql-null")

    (documentation

      (p "The API of SQLite changed significantly from version 2.x to 3.x. These are new bindings to the modified API, which are reasonably complete -- most procedures that take callback arguments are missing, though.")
      (p "For in-depth information on the functionality of the routines and general information you should consult the " (url "http://www.sqlite.org/" "SQLite documentation") " as well as this manual.")

      (subsection "Exceptions"

        (p "Unless otherwise indicated, all procedures and methods in this egg may throw an exception of the kind " (tt "(exn sqlite3)") " if something goes wrong. This exception will contain a " (tt "status") " property indicating the return value of the operation that failed:"
        (table
          (tr (th "Symbol") (th "Meaning"))
          #;(tr (td (tt "ok")) (td "Successful result"))
          (tr (td (tt "error")) (td "SQL error or missing database "))
          (tr (td (tt "internal")) (td "An internal logic error in SQLite "))
          (tr (td (tt "permission")) (td "Access permission denied "))
          (tr (td (tt "abort")) (td "Callback routine requested an abort "))
          (tr (td (tt "busy")) (td "The database file is locked "))
          (tr (td (tt "locked")) (td "A table in the database is locked "))
          (tr (td (tt "no-memory")) (td "A malloc() failed "))
          (tr (td (tt "read-only")) (td "Attempt to write a readonly database "))
          (tr (td (tt "interrupt")) (td "Operation terminated by sqlite-interrupt() "))
          (tr (td (tt "io-error")) (td "Some kind of disk I/O error occurred "))
          (tr (td (tt "corrupt")) (td "The database disk image is malformed "))
          (tr (td (tt "not-found")) (td "(Internal Only) Table or record not found "))
          (tr (td (tt "full")) (td "Insertion failed because database is full "))
          (tr (td (tt "cant-open")) (td "Unable to open the database file "))
          (tr (td (tt "protocol")) (td "Database lock protocol error "))
          (tr (td (tt "empty")) (td "(Internal Only) Database table is empty "))
          (tr (td (tt "schema")) (td "The database schema changed "))
          (tr (td (tt "too-big")) (td "Too much data for one row of a table "))
          (tr (td (tt "constraint")) (td "Abort due to contraint violation "))
          (tr (td (tt "mismatch")) (td "Data type mismatch "))
          (tr (td (tt "misuse")) (td "Library used incorrectly "))
          (tr (td (tt "no-lfs")) (td "Uses OS features not supported on host "))
          (tr (td (tt "authorization")) (td "Authorization denied"))
          #;(tr (td (tt "row")) (td (tt "step!") " has another row ready "))
          (tr (td (tt "done")) (td (tt "step!") " has finished executing, so no further data is ready")))))

      (subsection "Abstract data types"

        (procedure ("(database? OBJECT) " (& "rArr") " BOOLEAN")
          (p "Checks whether a value represents an SQLite database."))
        (procedure ("(error-database LOCATION OBJECT [ARGUMENT-NAME]) " (& "rArr") " VOID")
          (p "Raises a type error saying that a database was expected instead "
             "of the given value."))
        (procedure ("(check-database LOCATION OBJECT [ARGUMENT-NAME]) " (& "rArr") " VOID")
          (p "Raises a type error like " (tt "error-database") " does, unless "
             "the given value satisfies " (tt "database?")))

        (procedure ("(statement? OBJECT) " (& "rArr") " BOOLEAN")
          (p "Checks whether the value " (tt "v") " represents an SQL statement."))
        (procedure ("(error-statement LOCATION OBJECT [ARGUMENT-NAME]) " (& "rArr") " VOID")
          (p "Raises a type error saying that a statement was expected instead of the given value."))
        (procedure ("(check-statement LOCATION OBJECT [ARGUMENT-NAME]) " (& "rArr") " VOID")
          (p "Raises a type error like " (tt "error-statement") " does, unless the given value satisfies " (tt "statement?"))))

      (subsection "Managing databases"

        (procedure ("(open-database PATH) " (& "rArr") " DATABASE")
          (p "Opens the indicated database file and returns a database object for it.")
          (p "The given path is subject to the same special expansion as paths passed to " (tt "open-input-file") " and similar procedures."))

        (procedure ("(define-collation DATABASE NAME [PROC]) " (& "rArr") " VOID")
          (p "If a procedure is given, registers a new collation sequence identified by " (tt "name") " for use in the context of database handle " (tt "db") ". If no procedure is passed, the collation sequence with the given name is removed.")
          (p (tt "PROC") " should have the signature " (tt "(PROC STRING STRING) " (& "rArr") " FIXNUM") ". It should return a negative number if the first argument sorts before the second, a positive number if the second sorts before the first and zero if they are equal.")
          (p "As " (tt "PROC") " will be called in a callback context from within " (tt "step!") ", safety measures are installed to avoid throwing any exceptions, invoking continuations or returning invalid values from it. Attempts to do so will result in a " (tt "0") " return value and warning messages."))

        (definition
          (signatures
            (signature "procedure" ("(define-function DATABASE NAME N PROC) " (& "rArr") " VOID"))
            (signature "procedure" ("(define-function DATABASE NAME N STEP-PROC SEED [FINAL-PROC]) " (& "rArr") " VOID")))
          (p "Registers a new SQL function identified by " (tt "NAME") " for use in the context of the given database handle. If " (tt "STEP-PROC") " and " (tt "SEED") " are given, the new function becomes an aggregate function. Once registered, functions cannot be deleted.")
          (p (tt "N") " is the number of parameters the new SQL function takes or " (tt "-1") " to allow any number of arguments.")
          (p (tt "PROC") " should have the signature " (tt "(PROC . PARAMS) " (& "rArr") " OBJECT") ". It is called with the " (tt "N") " parameters given to the SQL function converted into Scheme objects like by " (tt "column-data") ". The return value is converted into an SQLite data object like by " (tt "bind!") ". A return value satisfying " (tt "sql-null?") " corresponds to " (tt "NULL") " in SQLite.")
          (p (tt "STEP-PROC") " should have the signature " (tt "(STEP-PROC SEED . PARAMS) " (& "rArr") " SEED") ". It is called with the parameters given to the SQL function for every row being processed. The seed value passed is initially the one given as an argument to " (tt "define-function") "; for subsequent calls it is the last value returned by " (tt "STEP-PROC") " and after completion of " (tt "FINAL-PROC") " it will be the initial value again.")
          (p (tt "FINAL-PROC") " should have the signature " (tt "(FINAL-PROC SEED) " (& "rArr") " OBJECT") " and transforms the last seed value into the value to be returned from the aggregate function. If it is not explicitly specified, " (tt "STEP-PROC") " defaults to the identity function.")
          (p "As " (tt "PROC") ", " (tt "STEP-PROC") " and " (tt "FINAL-PROC") " will be called in a callback context from within " (tt "step!") ", safety measures are installed to avoid throwing any exceptions, invoking continuations or returning invalid values from them. Attempts to do such things will result in " (tt "NULL") " return values and warning messages."))

        (procedure ("(set-busy-handler! DATABASE PROC) " (& "rArr") " VOID")
          (p "Installs the supplied procedure as the application's busy handler, or removes it if " (tt "#f") ".  When the database returns a busy error code, the egg will invoke this handler repeatedly until it returns " (tt "#f") ".  The handler will be called with arguments " (tt "DATABASE") " and " (tt "COUNT") " (number of times invoked for the same operation).")
          (p "As " (tt "PROC") " is not called in a callback context, it is legal to invoke captured continuations, and it is safe in the presence of multiple threads.  In general, this handler should give up at some point to avoid possible deadlock.")
          (p "For an example handler, see the code of " (tt "make-busy-timeout") "."))
        (procedure ("(make-busy-timeout MS) " (& "rArr") " PROC")
          (p "Returns a handler suitable for use with " (tt "set-busy-handler!") ". It polls in increasing intervals until the timeout in milliseconds is reached. The handler is non-blocking.")
          (pre ";; Example:
(define open-database/timeout
  (let ((handler (make-busy-timeout 2000)))
    (lambda (db-name)
      (let ((db (open-database db-name)))
  (set-busy-handler! db handler)
  db))))"))

        (procedure ("(interrupt! DATABASE) " (& "rArr") " VOID")
          (p "Cancels any running database operation as soon as possible.")
          (p "This function is always successful and never throws an exception."))

        (procedure ("(auto-committing? DATABASE) " (& "rArr") " BOOLEAN")
          (p "Checks whether the database is currently in auto committing mode, i.e. no transaction is currently active.")
          (p "This function always returns a state and never throws an exception."))

        (procedure ("(change-count DATABASE [TOTAL]) " (& "rArr") " CARDINAL-INTEGER")
          (p "Returns the number of rows changed by the last statement (if " (tt "(not TOTAL)") ", which is the default) or since the database was opened (if " (tt "TOTAL") ").")
          (p "This function always returns a count and never throws an exception."))

        (procedure ("(last-insert-rowid DATABASE) " (& "rArr") " INTEGER")
          (p "Returns the row ID of the last row inserted in " (tt "db") ".")
          (p "This function always returns a number and never throws an exception."))

        (definition
          (signatures
            (signature "procedure" ("(finalize! DATABASE [FINALIZE-STATEMENTS?]) " (& "rArr") " VOID"))
            (signature "procedure" ("(finalize! STATEMENT) " (& "rArr") " VOID")))
          (p "Closes the given database or finalizes the given statement.")
          (p "Every statement must be finalized to free its resources and discard it before the database itself can be finalized. However, if " (tt "FINALIZE-STATEMENTS?") " is not " (tt "#f") ", finalizing the database triggers automatic finalization of all statements first. " (tt "FINALIZE-STATEMENTS?") " defaults to " (tt "#f") ".")
          (p "Note that both the SQLite3 egg and the SQLite3 library itself try to detect the use of already finalized statement or database handles in API calls, but the detection is not always possible and you might crash the program by using an already finalized handle.")))

      (subsection "Managing statements"

        (procedure ("(prepare DATABASE SQL) " (& "rArr") " STATEMENT, SQL")
          (p "Compiles the first SQL statement in " (tt "SQL") " and returns a statement and the tail of the SQL code, which was not compiled (or an empty string)."))

        (procedure ("(source-sql STATEMENT) " (& "rArr") " STRING")
          (p "Retrieves the SQL source code of a statement."))

        (procedure ("(column-count STATEMENT) " (& "rArr") " CARDINAL-INTEGER")
          (p "Can be applied to any statement and returns the number of columns it will return as results.")
          (p "This procedure always succeeds and never throws an exception."))

        (procedure ("(column-name STATEMENT I) " (& "rArr") " STRING")
          (p "Can be applied to any statement and returns the name of the column number " (tt "I") " (counting from 0) as a string or " (tt "#f") " if the column has no name.")
          (p "This procedure always succeeds and never throws an exception."))

        (procedure ("(column-declared-type STATEMENT I) " (& "rArr") " STRING")
          (p "Can be applied to any statement and returns the declared type (as given in the " (tt "CREATE") " statement) of the column number " (tt "I") " (counting from 0) as a string or " (tt "#f") " if the column has no declared type.")
          (p "This procedure always succeeds and never throws an exception."))

        (procedure ("(bind-parameter-count STATEMENT) " (& "rArr") " CARDINAL-INTEGER")
          (p "Can be applied to any statement and returns the number of free parameters that can be bound in the statement.")
          (p "This procedure always succeeds and never throws an exception."))

        (procedure ("(bind-parameter-index STATEMENT NAME) " (& "rArr") " CARDINAL-INTEGER")
          (p "Can be applied to any statement and returns the index of the bindable parameter called " (tt "NAME") " or " (tt "#f") " if no such parameter exists.")
          (p "This procedure always succeeds and never throws an exception."))

        (procedure ("(bind-parameter-name STATEMENT I) " (& "rArr") " STRING")
          (p "Can be applied to any statement and returns the name of the bindable parameter number " (tt "I") " (counting from 0) or " (tt "#f") " if no such parameter exists or the parameter has no name.")
          (p "This procedure always succeeds and never throws an exception."))

        (procedure ("(bind! STATEMENT I OBJECT) " (& "rArr") " VOID")
          (p "Can be applied to any statement to bind its free parameter number " (tt "I") " (counting from 0) to the given value. Scheme types of the value map to SQLite types as follows:"
          (table
            (tr (th "Scheme type") (th "SQLite type"))
            (tr (td (tt "boolean?")) (td (tt "integer: #t = 1, #f = 0")))
            (tr (td (tt "fixnum?")) (td (tt "integer")))
            (tr (td (tt "real?")) (td (tt "float")))
            (tr (td (tt "string?")) (td (tt "text")))
            (tr (td (tt "blob?")) (td (tt "blob")))
            (tr (td (tt "sql-null?")) (td (tt "null")))))
          (p "Unless there is internal trouble in SQLite, this method should always succeeds and never throw an exception. For invalid parameter indices the method just silently does nothing."))

        (procedure ("(bind-parameters! STATEMENT . PARAMETERS) " (& "rArr") " VOID")
          (p "Resets the statement and binds all its free parameters.")
          (p "In addition to just listing the values to bind to the statement's parameters in sequence, you may specify parameters prefixed by keywords that are resolved to parameter indices by prefixing their names with " (tt "\":\"") " and resolving them using " (tt "bind-parameter-index") "."))

        (procedure ("(step! STATEMENT) " (& "rArr") " BOOLEAN")
          (p "Single-steps the execution of " (tt "STATEMENT") " and returns " (tt "#t") " if a result row was produced, " (tt "#f") " if no further results are available as the statement has been stepped through. This procedure must be called at least once before any results can be retrieved from the statement."))

        (procedure ("(column-type STATEMENT I) " (& "rArr") " SYMBOL")
          (p "Can be applied to a statement that has just been stepped (otherwise it returns " (tt "#f") ") and returns the SQLite type of the result column number " (tt "I") " (counting from 0) as a symbol.")
          (p "The return value can be one of the symbols " (tt "null") ", " (tt "integer") ", " (tt "float") ", " (tt "text") " or " (tt "blob") ".")
          (p "This procedure always succeeds and never throws an exception."))

        (procedure ("(column-data STATEMENT I) " (& "rArr") " OBJECT")
          (p "Can be applied to a statement that has just been stepped. Consults " (tt "column-type") " and " (tt "column-declared-type") " to determine the type of the indicated column and to return its data as an appropriate Scheme object:"
          (table
            (tr (th "SQLite type") (th "Scheme type"))
            (tr (td (tt "integer") ", declared " (tt "\"bool\"")) (td (tt "boolean?")))
            (tr (td (tt "integer")) (td (tt "integer?")))
            (tr (td (tt "float")) (td (tt "real?")))
            (tr (td (tt "text")) (td (tt "string?")))
            (tr (td (tt "blob")) (td (tt "blob?")))
            (tr (td (tt "null")) (td (tt "sql-null?")))))
          (p "The declared type of a column is considered to be boolean if the type declaration contains the character sequence \"bool\" anywhere, ignoring case.")
          (p "This procedure always succeeds and never throws an exception."))

        (procedure ("(reset! STATEMENT) " (& "rArr") " VOID")
          (p "Can be applied to any statement and resets it such that execution using " (tt "step!") " will perform all operations of the statement again.")))

      (subsection "Simple statement interface"

        (procedure ("(call-with-temporary-statements PROC DATABASE . SQLS) " (& "rArr") " OBJECT")
          (p "Compiles the SQL sources into statements in the context of " (tt "DATABASE") ", applies " (tt "PROC") " to these statements and returns " (tt "PROC") "'s result. The statements are created and finalized in " (tt "dynamic-wind") " entry and exit blocks around the application of " (tt "PROC") "."))

        (definition
          (signatures
            (signature "procedure" ("(execute STATEMENT . PARAMETERS) " (& "rArr") " VOID"))
            (signature "procedure" ("(execute DATABASE SQL . PARAMETERS) " (& "rArr") " VOID")))
          (p "(Compiles the given SQL), resets the statement, binds the statement's free parameters and executes the statement ignoring possible results from it."))

        (definition
          (signatures
            (signature "procedure" ("(update STATEMENT . PARAMETERS) " (& "rArr") " CARDINAL-INTEGER"))
            (signature "procedure" ("(update DATABASE SQL . PARAMETERS) " (& "rArr") " CARDINAL-INTEGER")))
          (p "(Compiles the given SQL), resets the statement, binds the statement's free parameters and executes the specified statement ignoring possible results from it, returning the result of applying " (tt "change-count") " to the affected database after the execution of the statement instead."))

        (definition
          (signatures
            (signature "procedure" ("(first-result STATEMENT . PARAMETERS) " (& "rArr") " OBJECT"))
            (signature "procedure" ("(first-result DATABASE SQL . PARAMETERS) " (& "rArr") " OBJECT")))
          (p "(Compiles the given SQL), resets the statement, binds the statement's free parameters and single-steps the statement once returning the value of the first column in the first result row. Resets the statement again just before returning.")
          (p "If the given statement does not yield any results, an " (tt "(exn sqlite3)") " is thrown with the " (tt "status") "-property set to " (tt "done") "."))

        (definition
          (signatures
            (signature "procedure" ("(first-row STATEMENT . PARAMETERS) " (& "rArr") " LIST"))
            (signature "procedure" ("(first-row DATABASE SQL . PARAMETERS) " (& "rArr") " LIST")))
          (p "(Compiles the given SQL), resets the statement, binds the statement's free parameters and single-steps the statement once returning all columns in the first result row as a list.")
          (p "If the given statement does not yield any results, an " (tt "(exn sqlite3)") " is thrown with the " (tt "status") "-property set to " (tt "done") "."))

        (definition
          (signatures
            (signature "procedure" ("(fold-row PROC INIT STATEMENT . PARAMETERS) " (& "rArr") " OBJECT"))
            (signature "procedure" ("(fold-row PROC INIT DATABASE SQL . PARAMETERS) " (& "rArr") " OBJECT")))
          (p "(Compiles the given SQL), resets the statement, binds the statement's free parameters and executes it step by step. After each step, the column values of the current result row are retrieved and " (tt "PROC") " is applied to the current folded value, set to " (tt "INIT") " in the first step, and the column values. The result of the application becomes the new folded value."))

        (definition
          (signatures
            (signature "procedure" ("(for-each-row PROC STATEMENT . PARAMETERS) " (& "rArr") " VOID"))
            (signature "procedure" ("(for-each-row PROC DATABASE SQL . PARAMETERS) " (& "rArr") " VOID")))
          (p "(Compiles the given SQL), resets the statement, binds the statement's free parameters and executes it step by step. After each step, the column values of the current result row are retrieved and " (tt "PROC") " is applied to them. The results of this application are discarded."))

        (definition
          (signatures
            (signature "procedure" ("(map-row PROC STATEMENT . PARAMETERS) " (& "rArr") " LIST"))
            (signature "procedure" ("(map-row PROC DATABASE SQL . PARAMETERS) " (& "rArr") " LIST")))
          (p "(Compiles the given SQL), resets the statement, binds the statement's free parameters and executes it step by step. After each step, the column values of the current result row are retrieved and " (tt "PROC") " is applied to them. The results of these applications are collected into a list.")))

      (subsection "Utility functions"

        (procedure ("(with-transaction DATABASE THUNK [TYPE]) " (& "rArr") " OBJECT")
          (p "Runs " (tt "THUNK") " within the scope of a transaction on the database and returns the return value from " (tt "THUNK") ".")
          (p "The transaction is committed upon exit from " (tt "THUNK") " if " (tt "THUNK") " returns a true value. If " (tt "THUNK") " returns a false value or throws an exception, the transaction is rolled back.")
          (p "The " (tt "TYPE") " of the transaction can be specified as one of the symbols " (tt "deferred") "(the default), " (tt "immediate") " or " (tt "exclusive") "."))

        (procedure ("(sql-complete? SQL) " (& "rArr") " BOOLEAN")
          (p "Checks whether " (tt "SQL") " comprises at least one complete SQL statement."))

        (procedure ("(enable-shared-cache! BOOLEAN) " (& "rArr") " VOID")
          (p "Enables (or disables) the sharing of the database cache and schema data structures between connections to the same database."))

        (procedure ("(database-version) " (& "rArr") " STRING")
          (p "Returns a string identifying the version of SQLite in use."))

        (procedure ("(database-memory-used) " (& "rArr") " CARDINAL-INTEGER")
          (p "Returns the amount of memory currently in use by the database engine."))

        (procedure ("(database-memory-highwater [RESET?]) " (& "rArr") " CARDINAL-INTEGER")
          (p "Returns the maximum amount of memory that was in use by the database engine since the counter was last reset or since the program started. Resets the counter if " (tt "RESET?") " is not " (tt "#f") ". " (tt "RESET?") " defaults to " (tt "#f") "."))))

    (history
      (version "3.5.1" "Corrected parameter error checks for define-function")
      (version "3.5.0" "Switched to less error prone internal C API")
      (version "3.4.0" "Added optional automatic statement finalization when a database is finalized")
      (version "3.3.1" "Added bindings for database memory statistics")
      (version "3.3.0" "Switched to using " (tt "(sql-null)") " for " (tt "NULL") " values")
      (version "3.2.1" "Added a test suite")
      (version "3.2.0" "Removed the unsafe busy handler and timeout APIs, since a safe API exists")
      (version "3.1.0" (tt "bind-parameters!") " can now handle keyword arguments")
      (version "3.0.0" "Ported to CHICKEN 4, removed dependencies on TinyCLOS and EasyFFI")
      (version "2.0.8" "Add busy handler callbacks; ensure finalize! is called on exception. [Jim Ursetto]")
      (version "2.0.7" "Restore error reporting. [Jim Ursetto]")
      (version "2.0.6" "Add " (tt "enable-shared-cache!") ", requires 3.3.0 or later. [Jim Ursetto]")
      (version "2.0.5" "Added some <boolean> support. Change for NULL (<void>) handling. [Kon Lovett]")
      (version "2.0.4" "Added " (code "sqlite3:fold-row") " & " (code "sqlite3:bind-parameters!") ". Fix for introduced bug in " (code "sqlite3:changes") ". [Kon Lovett]")
      (version "2.0.3" "Added " (code "sqlite3:null-value") ", " (code "sqlite3:null-value?") ", and " (code "sqlite3:null") ". [Kon Lovett]")
      (version "2.0.2" "Use of extended " (tt "define-foreign-enum") ". Removed deprecated " (tt "pointer") " use. [Kon Lovett]")
      (version "2.0.1" "Deprecated " (tt "<byte-vector>") ", use " (tt "<blob>") " [Kon Lovett]")
      (version "2.0.0" "Now using " (tt "(void)") " to represent " (tt "NULL"))
      (version "1.5.9" "dll extension not used anymore in newer chickens [felix]")
      (version "1.5.8" "Update for synch 1.3 [Kon Lovett]")
      (version "1.5.7" "Updated compiler flags to pull in tinyclos and easyffi")
      (version "1.5.6" "Replaced deprecated synch operations [Kon Lovett]")
      (version "1.5.5" "Correction in the documentation, added sqlite3:with-transaction")
      (version "1.5.4" "Typo fixed thanks to the new imports checking code")
      (version "1.5.3" "Proper multithreading locks for the callback code")
      (version "1.5.2" "Code cleanups")
      (version "1.5.1" "Potential memory leaks removed")
      (version "1.5.0" "Support for user defined collation sequences and functions has been added")
      (version "1.4.0" "Stale statement handles due to schema changes are now automagically recompiled to keep them valid")
      (version "1.3.1" "Several small routines added")
      (version "1.3.0" "Special hacks removed as CVS version of SQLite3.3.4 has been fixed")
      (version "1.2.0" "Special hacks to deal with \"valid\" " (tt "NULL") " statements")
      (version "1.1.4" "Tightened security measures against " (tt "#f") " pointers")
      (version "1.1.3" "Integers not fitting in a fixnum are now read correctly from the database [thanks to Zbigniew]")
      (version "1.1.2" "Setup file patched to call compiled output .dll on Windows")
      (version "1.1.1" "All procedures now reset prepared statements where you would do that by hand anyway")
      (version "1.1.0" "Promoted " (tt "sqlite3:call-with-temporary-statement") " to " (tt "sqlite3:call-with-temporary-statements") " and fixed a really stupid coding mistake in " (tt "sqlite3:changes"))
      (version "1.0.3" "Fixed C compiler warnings")
      (version "1.0.2" "Added a typecheck for increased safety")
      (version "1.0.1" "Fixed type mistakes in the source")
      (version "1.0.0" "Initial release") )

     (license #<<EOS
Copyright (c) 2005-2009, Thomas Chust <chust@web.de>.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

  Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer. Redistributions in
  binary form must reproduce the above copyright notice, this list of
  conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution. Neither the name of the
  author nor the names of its contributors may be used to endorse or
  promote products derived from this software without specific prior
  written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
EOS
) ) ) )

(eggdoc->html doc
  (append (eggdoc:make-stylesheet doc)
    `((method *macro* . ,eggdoc:make-defsig))))
